@@ -33,21 +33,34 @@ def buildurl(*args,**kw):
     if len(kw):
         url += "?" + urlencode(kw)
     return url
-        
 
 
 from lino.utils.xmlgen import html as xghtml
+from lino.utils import AttrDict
+#~ from lino import pypi_info 
+#~ from lino.pypi_info import pypi_info
 
-__version__ = "1.5.6"
+execfile(os.path.join(os.path.dirname(__file__),'version.py'))
+
+#~ execfile(os.path.join(os.path.dirname(__file__),'pypi_info.py'))
+#~ x = file(os.path.join(os.path.dirname(__file__),'pypi_info.py')).read().strip()
+#~ pypi_info = AttrDict(eval(x))
+#~ pypi_info = AttrDict(pypi_info)
+
+
+#~ __version__ = pypi_info.version
+#~ __version__ = "1.5.6"
 """
 Lino version number. 
 """
 
 __author__ = "Luc Saffre <luc.saffre@gmx.net>"
+#~ __author__ = pypi_info.author
 
 #~ __url__ = "http://lino.saffre-rumma.net"
 #~ __url__ = "http://code.google.com/p/lino/"
 __url__ = "http://www.lino-framework.org"
+#~ __url__ = pypi_info.url
 
 
 __copyright__ = """\
@@ -56,7 +69,6 @@ This software comes with ABSOLUTELY NO WARRANTY and is
 distributed under the terms of the GNU General Public License.
 See file COPYING.txt for more information."""
 
-
 gettext = lambda s: s
 
 def language_choices(*args):
@@ -107,25 +119,31 @@ NOT_FOUND_MSG = '(not installed)'
 
 class Lino(object):
     """
-    Base class for the Lino Application instance stored in :setting:`LINO`.
+    Base class for the Lino instance stored in :setting:`LINO`.
+    Lino applications add one setting `LINO` 
+    which is an instance of the :class:`lino.Lino` class.
+    This simple trick brings inheritance to the settings and lets us define 
+    methods.
     
     This class is first defined in :mod:`lino`, 
     then usually subclassed by the application developer
     (e.g. :mod:`lino.apps.cosi.Lino`),
-    which is imported into your local :xfile:`settings.py`,
-    where you may subclass it another time.
+    then imported into your local :xfile:`settings.py`,
+    where you may subclass it another time before 
+    finally instantiating it, and assigning it to 
+    the :setting:`LINO` variable.
     
-    To activate your local Lino settings, 
-    you must instantiate it and store the instance 
-    in the :setting:`LINO` variable of your :xfile:`settings.py`::
+    Instiatiation is always the same line of code::
     
       LINO = Lino(__file__,globals())
       
-    With the parameters `__file__` and `globals()` you give Lino information 
-    about your local settings. 
-    Lino will modify the following Django settings
-    :setting:`FIXTURE_DIRS`, :setting:`MEDIA_ROOT` and :setting:`TEMPLATE_DIRS` 
-    .
+    With the parameters `__file__` and `globals()` you give Lino 
+    information about your local settings (where they are in the file 
+    system), and the possibility to modify your Django settings.
+    
+    Lino will modify the following Django settings:
+    :setting:`FIXTURE_DIRS`, :setting:`MEDIA_ROOT` 
+    and :setting:`TEMPLATE_DIRS`,...
     
     """
     
@@ -174,6 +192,15 @@ class Lino(object):
     
     use_spinner = False # doesn't work. leave this to False
     
+    #~ django_admin_prefix = '/django' 
+    django_admin_prefix = None
+    """
+    The prefix to use for Django admin URLs.
+    Leave this unchanged as long as :dmodules
+    oc:`/ticket/70` is not solved.
+    Leave this unchanged as long as :doc:`/ticket/70` is not solved.
+    """
+    
     plain_prefix = '/plain' 
     """
     The prefix to use for the "plain html" URLs.
@@ -207,6 +234,8 @@ class Lino(object):
     https://code.djangoproject.com/wiki/BackwardsIncompatibleChanges#ChangedthewayURLpathsaredetermined
     """
     
+    use_extjs = True
+    
     
     extjs_root = None
     """
@@ -215,6 +244,18 @@ class Lino(object):
     directory has no symbolic link to the ExtJS root directory.
     """
     
+    extjs_base_url = "http://extjs-public.googlecode.com/svn/tags/extjs-3.3.1/release/"
+    """
+    The default value points to the 
+    `extjs-public <http://code.google.com/p/extjs-public/>`_
+    repository and thus requires the clients to have an internet connection.
+    
+    Set this to None if you want to serve ExtJS from your server
+    (which requires a symbolic link "extjs" in your media directory
+    or :attr:`extjs_root` pointing to the local directory 
+    where ExtJS 3.3.1 is installed).
+    """
+    
     bootstrap_root = None
     """
     Path to the Jasmine root directory. 
@@ -281,13 +322,6 @@ class Lino(object):
     title = None
     #~ domain = "www.example.com"
     
-    pypi_name = None
-    """
-    The PyPI name of this application. 
-    Used in :xfile:`setup.py` file as the `name` argument to 
-    `setuptools.setup() <http://guide.python-distribute.org/creation.html>`_.
-    """
-    
     short_name = None # "Unnamed Lino Application"
     """
     Used as display name to end-users at different places.
@@ -337,22 +371,22 @@ class Lino(object):
     """
     
     #~ person_model = None
-    person_model = "contacts.Person"
-    """
-    If your application uses :model:`lino.modlib.contacts`,
-    set this to a string "applabel.Modelname" which identifies 
-    your Person model (which should inherit from
-    :class:`lino.modlib.contacts.models.Person`).
-    """
+    #~ person_model = "contacts.Person"
+    #~ """
+    #~ If your application uses :model:`lino.modlib.contacts`,
+    #~ set this to a string "applabel.Modelname" which identifies 
+    #~ your Person model (which should inherit from
+    #~ :class:`lino.modlib.contacts.models.Person`).
+    #~ """
     
     #~ company_model = None
-    company_model = "contacts.Company"
-    """
-    If your application uses :model:`lino.modlib.contacts`,
-    set this to a string "applabel.Modelname" which identifies 
-    your Company model (which should inherit from
-    :class:`lino.modlib.contacts.models.Company`).
-    """
+    #~ company_model = "contacts.Company"
+    #~ """
+    #~ If your application uses :model:`lino.modlib.contacts`,
+    #~ set this to a string "applabel.Modelname" which identifies 
+    #~ your Company model (which should inherit from
+    #~ :class:`lino.modlib.contacts.models.Company`).
+    #~ """
     
     project_model = None
     """
@@ -381,7 +415,7 @@ class Lino(object):
     
     anonymous_user_profile = '000'
     """
-    The UserProfile (or rather it's value) to assigned to anonymous user.
+    The UserProfile to be assigned to anonymous user.
     
     """
     
@@ -701,6 +735,9 @@ class Lino(object):
     If you change this setting, you also need to override :meth:`parse_datetime`.
     """
     
+    #~ default_number_format_extjs = '0,000.00/i'
+    default_number_format_extjs = '0,00/i'
+    
     uppercase_last_name = False
     """
     Whether last name of persons should be printed with uppercase letters.
@@ -763,6 +800,12 @@ class Lino(object):
     by :class:`lino.modlib.cal.models.EventOwner`.
     """
     
+    #~ mergeable_models = []
+    #~ """
+    #~ A list of models that should have a "Merge" action
+    #~ (see :mod:`lino.mixins.mergeable`).
+    #~ """
+    
     override_modlib_models = []
     """
     A list of names of modlib models which are being 
@@ -790,7 +833,7 @@ class Lino(object):
     
     """
     
-    modules = None
+    modules = AttrDict()
     """
     A shortcut to access all installed models and actors.
     Read-only. Applications should not set this. 
@@ -855,6 +898,7 @@ class Lino(object):
         self._setup_done = False
         #~ self._response = None
         self.django_settings = django_settings
+        self.GFK_LIST = []
         
         
         installed_apps = tuple(self.get_installed_apps())
@@ -922,7 +966,30 @@ class Lino(object):
             TEMPLATE_LOADERS=tuple(
                 ['lino.core.web.Loader']
                 ))
-
+                
+        #~ tl = [
+            #~ 'django.template.loaders.filesystem.Loader',
+            #~ 'django.template.loaders.app_directories.Loader',
+            #~ #     'django.template.loaders.eggs.load_template_source',
+        #~ ]
+        #~ django_settings.update(TEMPLATE_LOADERS = tuple(tl))
+                
+                
+        tcp = []
+        if self.user_model == 'auth.User':
+            django_settings.update(LOGIN_URL = '/accounts/login/')
+            django_settings.update(LOGIN_REDIRECT_URL = "/")
+            tcp += [ 'django.contrib.auth.context_processors.auth' ]
+            
+        tcp += [
+                'django.core.context_processors.debug',
+                'django.core.context_processors.i18n',
+                'django.core.context_processors.media',
+                'django.core.context_processors.static',
+            #    'django.core.context_processors.request',
+                #~ 'django.contrib.messages.context_processors.messages',
+        ]
+        django_settings.update(TEMPLATE_CONTEXT_PROCESSORS = tuple(tcp))
 
         self.startup_time = datetime.datetime.now()
         
@@ -1209,7 +1276,7 @@ class Lino(object):
         
     def get_site_menu(self,ui,profile):
         """
-        Return this site's main menu for the given user. 
+        Return this site's main menu for the given UserProfile. 
         Must be a :class:`lino.core.menus.Toolbar` instance.
         Applications usually should not need to override this.
         """
@@ -1221,10 +1288,9 @@ class Lino(object):
         #~ url = self.admin_url
         #~ if not url: 
             #~ url = "/"
+        #~ main.add_url_button(url,label=_("Home"))
         #~ url = "javascript:Lino.close_all_windows()"
         #~ main.add_url_button(url,label=_("Home"))
-        main.add_item('home',_("Home"),javascript="Lino.close_all_windows()")
-        
         return main
         
     def setup_quicklinks(self,ar,m):
@@ -1775,11 +1841,12 @@ class Lino(object):
         This method is expected to return or yield the list of strings 
         to be stored into Django's :setting:`INSTALLED_APPS` setting.
         """
+        yield 'lino'
         if self.user_model is not None and self.remote_user_header is None:
             yield 'django.contrib.sessions' # 20121103
-        #~ 'django.contrib.sites',
+        if self.django_admin_prefix:
+            yield 'django.contrib.admin'
         #~ 'django.contrib.markup',
-        yield 'lino'
         yield 'lino.modlib.about'
         #~ if self.admin_prefix:
             #~ yield 'lino.modlib.pages'
@@ -1801,8 +1868,13 @@ class Lino(object):
     def build_plain_url(self,*args,**kw):
         return self.plain_prefix + buildurl(*args,**kw)
         
+    def build_extjs_url(self,url):
+        if self.extjs_base_url:
+            return self.extjs_base_url + url
+        return self.build_media_url('extjs',url)
+        
 
-    def get_urls(self):
+    def unused_get_urls(self):
         return []
 
     #~ def get_sidebar_html(self,request=None,node=None,**context):
@@ -1813,4 +1885,15 @@ class Lino(object):
     """
     Width of the sidebar in 1/12 of total screen width.
     Meaningful values are 0 (no sidebar), 2 or 3.
-    """
\ No newline at end of file
+    """
+    
+    
+    def get_generic_related(self,obj):
+        from django.contrib.contenttypes.models import ContentType
+        for gfk in self.GFK_LIST:
+            ct = ContentType.objects.get_for_model(gfk.model)
+            kw = dict()
+            kw[gfk.fk_field] = obj.pk
+            yield gfk, ct.get_all_objects_for_this_type(**kw)
+        
+            
\ No newline at end of file
