@@ -26,29 +26,30 @@ from django.conf import settings
 #~ from django.contrib.auth import models as auth
 #~ from django.contrib.sessions import models as sessions
 from django.contrib.contenttypes import models as contenttypes
+from django.contrib.contenttypes.models import ContentType
 from django.utils.encoding import force_unicode 
 
+
 #~ from django import forms
 from django.db import models
 from django.utils.translation import ugettext_lazy as _
 
-import lino
+#~ import lino
 from lino import mixins
 from lino import dd
 #~ from lino import commands
 from lino.mixins import printable
 from lino.utils import babel
 #~ from lino import choices_method, simple_choices_method
-from lino.core.modeltools import obj2str, sorted_models_list
-from lino.core.modeltools import resolve_field
+from lino.core.modeltools import obj2str, sorted_models_list, full_model_name
+from lino.core.modeltools import resolve_field, UnresolvedModel
 from lino.utils.choosers import chooser, get_for_field
-#~ from lino.modlib.users.models import UserLevels
 from lino.utils.restify import restify
 from lino.core import actions
+#~ from lino.core import changes
+from lino.utils.xmlgen import html as xghtml
 
-
-
-
+#~ from lino.core.changes import Change, Changes, ChangesByObject
 
 
 #~ class BuildLinoJS(dd.RowAction):
@@ -59,11 +60,11 @@ class BuildSiteCache(dd.RowAction):
     """
     label = _("Rebuild site cache")
     url_action_name = "buildjs"
-    def run(self,elem,rr):
-        rr.confirm(_("Are you sure?"))
+    def run(self,elem,ar):
+        #~ rr.confirm(_("Are you sure?"))
         #~ rr.confirm(_("Are you really sure?"))
-        rr.ui.build_site_cache(True)
-        return rr.ui.success_response(
+        ar.ui.build_site_cache(True)
+        return ar.success(
             """\
 Seems that it worked. Refresh your browser. 
 <br>
@@ -75,6 +76,7 @@ Please report any anomalies.""",
     
 
 
+    
 class SiteConfig(dd.Model):
     """
     This model should have exactly one instance, 
@@ -116,12 +118,12 @@ class SiteConfigs(dd.Table):
     """
     default_action = actions.ShowDetailAction()
     model = SiteConfig
-    required = dict(user_level='manager')
+    required = dd.required(user_level='manager')
     #~ default_action_class = dd.OpenDetailAction
     #~ has_navigator = False
     hide_top_toolbar = True
     #~ can_delete = perms.never
-    detail_template = """
+    detail_layout = """
     default_build_method
     # lino.ModelsBySite
     """
@@ -137,7 +139,7 @@ if settings.LINO.is_installed('contenttypes'):
       """
       model = contenttypes.ContentType
       
-      detail_template = """
+      detail_layout = """
       id name app_label model base_classes
       lino.HelpTextsByModel
       """
@@ -176,7 +178,8 @@ if settings.LINO.is_installed('contenttypes'):
           blank=True,null=True,format='plain')
       
       def __unicode__(self):
-          return self.content_type.app_label + '.' + self.content_type.name + '.' + self.field
+          return self.content_type.app_label + '.' + self.content_type.model + '.' + self.field
+          #~ return self.content_type.app_label + '.' + self.content_type.name + '.' + self.field
           
       @chooser(simple_values=True)
       def field_choices(cls,content_type):
@@ -193,7 +196,7 @@ if settings.LINO.is_installed('contenttypes'):
               for f in meta.many_to_many: l.append(f.name)
               for f in meta.virtual_fields: l.append(f.name)
               for a in model._lino_default_table.get_actions():
-                  l.append(a.name)
+                  l.append(a.action_name)
               l.sort()
           return l
           
@@ -203,7 +206,10 @@ if settings.LINO.is_installed('contenttypes'):
       @dd.virtualfield(models.CharField(_("Verbose name"),max_length=200))
       def verbose_name(self,request):
           #~ return unicode(self)
-          m = dd.resolve_model(self.content_type.app_label + '.' + self.content_type.name)
+          #~ m = dd.resolve_model(self.content_type.app_label + '.' + self.content_type.name)
+          m = self.content_type.model_class()
+          #~ if isinstance(m,UnresolvedModel):
+              #~ return str(m)
           de = m._lino_default_table.get_data_elem(self.field)
           if isinstance(de,models.Field):
               #~ return unicode(de.verbose_name)
@@ -226,7 +232,8 @@ if settings.LINO.is_installed('contenttypes'):
       
 
 
-#~ if settings.LINO.is_installed('users'):
+
+
 if settings.LINO.user_model:
 
     class TextFieldTemplate(mixins.AutoUser):
@@ -248,296 +255,31 @@ if settings.LINO.user_model:
             return self.name
             
     class TextFieldTemplates(dd.Table):
-        detail_template = """
+        detail_layout = """
         id name user
         description
         text
         """
         model = TextFieldTemplate
+        required = dd.required(user_groups='office')
 
     class MyTextFieldTemplates(TextFieldTemplates,mixins.ByUser):
         pass
         
 
 
-
-#~ class ModelsBySite(dd.VirtualTable):
-    #~ label = _("Models")
-    #~ column_names = "app name docstring rows"
-    #~ master = SiteConfig
-    
-    #~ slave_grid_format = 'html'    
-  
-    #~ @classmethod
-    #~ def get_data_rows(self,ar):
-        #~ for model in models.get_models():
-            #~ yield model
-                
-    #~ @dd.displayfield(_("app_label"))
-    #~ def app(self,obj,ar):
-        #~ return obj._meta.app_label
-        
-    #~ @dd.displayfield(_("name"))
-    #~ def name(self,obj,ar):
-        #~ return obj.__name__
-        
-    #~ @dd.displayfield(_("docstring"))
-    #~ def docstring(self,obj,ar):
-        #~ return obj.__doc__
-        
-    #~ @dd.requestfield(_("Rows"))
-    #~ def rows(self,obj,ar):
-        #~ return obj._lino_default_table.request(ar.ui,
-          #~ user=ar.get_user(),renderer=ar.renderer)
-        
-        
-class Models(dd.VirtualTable):
-    label = _("Models")
-    #~ column_defaults = dict(width=8)
-    #~ column_names = "app name verbose_name docstring rows"
-    column_names = "app name docstring rows"
-    #~ master = SiteConfig
-    detail_template = """
-    app name docstring rows
-    lino.FieldsByModel
-    """
-    
-    slave_grid_format = 'html'    
-  
-    @classmethod
-    def get_data_rows(self,ar):
-        for model in models.get_models():
-            yield model
-                
-    @dd.displayfield(_("app_label"))
-    def app(self,obj,ar):
-        return obj._meta.app_label
-        
-    @dd.displayfield(_("name"))
-    def name(self,obj,ar):
-        return obj.__name__
-        
-    #~ @dd.displayfield(_("verbose name"))
-    #~ def vebose_name(self,obj,ar):
-        #~ return unicode(obj._meta.vebose_name)
-        
-    @dd.displayfield(_("docstring"))
-    def docstring(self,obj,ar):
-        #~ return obj.__doc__
-        return restify(unicode(obj.__doc__))
-        
-    @dd.requestfield(_("Rows"))
-    def rows(self,obj,ar):
-        return obj._lino_default_table.request(ar.ui,
-          user=ar.get_user(),renderer=ar.renderer)
-
-
-class FieldsByModel(dd.VirtualTable):
-    label = _("Fields")
-    #~ master_key = "model"
-    master = Models
-    column_names = "name verbose_name help_text"
-    
-    @classmethod
-    def get_data_rows(self,ar):
-        model = ar.master_instance
-        if model:
-            for (fld,remote) in model._meta.get_fields_with_model():
-                yield fld
-                
-    @dd.displayfield(_("name"))
-    def name(self,fld,ar):
-        return fld.name
-        
-    @dd.displayfield(_("verbose name"))
-    def verbose_name(self,fld,ar):
-        return unicode(fld.vebose_name)
-        
-    @dd.displayfield(_("help text"))
-    def help_text(self,obj,ar):
-        #~ return obj.__doc__
-        return restify(unicode(obj.help_text))
-
-
-import inspect
-import types
-from lino.utils import AttrDict
-
-class Inspected(object):
-    def __init__(self,parent,prefix,name,value):
-        self.parent = parent
-        self.prefix = prefix
-        self.name = name
-        self.value = value
-
-class Inspector(dd.VirtualTable):
-    """
-    Shows a simplistic "inspector" which once helped me for debugging.
-    Needs more work to become seriously useful...
-    
-    """
-    label = _("Inspector")
-    required = dict(user_level='admin')
-    column_names = "i_name i_type i_value"
-    parameters = dict(
-      inspected=models.CharField(_("Inspected object"),max_length=100,blank=True),
-      show_callables=models.BooleanField(_("show callables"),default=False)
-      )
-    params_template = 'inspected show_callables'
-    #~ editable = False
-    #~ slave_grid_format = 'html'    
-  
-    @classmethod
-    def get_inspected(self,name):
-        #~ ctx = dict(settings=settings,lino=lino)
-        if not name:
-            return settings
-        try:
-            o = eval('settings.'+name)
-        except Exception,e:
-            o = e
-        return o
-        
-        #~ o = settings
-        #~ try:
-            #~ for ch in name.split('.'):
-                #~ o = getattr(o,ch)
-        #~ except Exception,e:
-            #~ o = e
-        #~ return o
-        
-    @classmethod
-    def get_data_rows(self,ar):
-        logger.info("20120210 %s, %s",ar.quick_search,ar.param_values.inspected)
-        
-        if ar.param_values.show_callables:
-            def flt(v): return True
-        else:
-            def flt(v): 
-                if isinstance(v,(
-                    types.FunctionType,
-                    types.GeneratorType,
-                    types.UnboundMethodType,
-                    types.UnboundMethodType,
-                    types.BuiltinMethodType,
-                    types.BuiltinFunctionType,
-                    )): 
-                    return False
-                return True
-          
-        
-        o = self.get_inspected(ar.param_values.inspected)
-        if isinstance(o,(list,tuple)):
-            for i,v in enumerate(o):
-                k = "[" + str(i) + "]"
-                yield Inspected(o,'',k,v)
-        elif isinstance(o,AttrDict):
-            for k,v in o.items():
-                yield Inspected(o,'.',k,v)
-        elif isinstance(o,dict):
-            for k,v in o.items():
-                k = "[" + repr(k) + "]"
-                yield Inspected(o,'',k,v)
-        else:
-            for k in dir(o):
-                if not k.startswith('__'):
-                    if not ar.quick_search or (ar.quick_search.lower() in k.lower()):
-                        v = getattr(o,k)
-                        if flt(v):
-                        #~ if not inspect.isbuiltin(v) and not inspect.ismethod(v):
-                            #~ if ar.param_values.show_callables or not inspect.isfunction(v):
-                            #~ if isinstance(v,types.FunctionType ar.param_values.show_callables or not callable(v):
-                            yield Inspected(o,'.',k,v)
-        #~ for k,v in o.__dict__.items():
-            #~ yield Inspected(o,k,v)
-            
-                
-    @dd.displayfield(_("Name"))
-    def i_name(self,obj,ar):
-        pv = dict()
-        if ar.param_values.inspected:
-            pv.update(inspected=ar.param_values.inspected+obj.prefix+obj.name)
-        else:
-            pv.update(inspected=obj.name)
-        #~ newreq = ar.spawn(ar.ui,user=ar.user,renderer=ar.renderer,param_values=pv)
-        newreq = ar.spawn(param_values=pv)
-        return ar.renderer.href_to_request(newreq,obj.name)
-        #~ return obj.name
-        
-    @dd.displayfield(_("Value"))
-    def i_value(self,obj,ar):
-        return cgi.escape(unicode(obj.value))
-        
-    @dd.displayfield(_("Type"))
-    def i_type(self,obj,ar):
-        return cgi.escape(str(type(obj.value)))
-        
-
-class AboutDetail(dd.FormLayout):
-    """
-    The Detail Layout for :class:`About`
-    """
-    window_size = (60,30)
-    main = """
-    versions:40x5 startup_time:30
-    lino.Models:70x10
-    """
-
-
-class About(mixins.EmptyTable):
-    """
-    A modal window displaying information about this server.
-    """
-    label = _("About") 
-    #~ hide_window_title = True
-    hide_top_toolbar = True
-    #~ window_size = (700,400)
-    #~ detail_layout = AboutDetail(window_size = (700,400))
-    detail_layout = AboutDetail()
-    #~ detail_template = """
-    #~ versions:40x5 startup_time:30
-    #~ lino.Models:70x10
-    #~ """
-    
-    #~ versions = dd.Constant(lino.welcome_html())
-    
-    do_build = BuildSiteCache()
-    
-    #~ @classmethod
-    #~ def setup_actions(self):
-        #~ super(About,self).setup_actions()
-        #~ self.add_action(BuildSiteCache())
-   
-    #~ @dd.constant(_("Versions"))
-    @dd.constant()
-    def versions(cls,ui):
-        return lino.welcome_html(ui)
-        
-    #~ @dd.displayfield(_("Versions"))
-    #~ def versions(self,obj,ar):
-        #~ return lino.welcome_html(ar.ui)
-        
-    #~ @dd.constantfield(_("Versions"))
-    #~ def versions(cls,self,req):
-        #~ return lino.welcome_html()
-        
-    @dd.virtualfield(models.DateTimeField(_("Server up since")))
-    def startup_time(cls,self,req):
-        return settings.LINO.startup_time
-    
-
-
 class Home(mixins.EmptyTable):
     """
     This is the "home page" or "welcome screen", the window to be displayed 
     when no other window is opened.
     """
+    required = dd.required()
     #~ debug_actions = True
     label = _("Home") 
     hide_window_title = True
     hide_top_toolbar = True
     #~ detail_layout = HomeDetail()
-    detail_template = """
+    detail_layout = """
     quick_links:80x1
     welcome
     """
@@ -552,14 +294,15 @@ class Home(mixins.EmptyTable):
         #~ return cal.tasks_summary(req.ui,req.get_user())
     
     @dd.virtualfield(dd.HtmlBox())
-    def quick_links(cls,self,req):
-        quicklinks = settings.LINO.get_quicklinks(self,req.get_user())
+    def quick_links(cls,self,ar):
+        quicklinks = settings.LINO.get_quicklinks(ar)
         if quicklinks.items:
-            #~ assert mi.params is None
-            return 'Quick Links: ' + ' '.join(
-              [req.ui.ext_renderer.action_href_js(mi.action) 
-                for mi in quicklinks.items]
-              )
+            chunks = []
+            for mi in quicklinks.items:
+                chunks.append(' ')
+                chunks.append(ar.ui.ext_renderer.window_action_button(
+                  ar,mi.bound_action))
+            return xghtml.E.p('Quick Links:',*chunks)
       
     #~ @dd.virtualfield(dd.HtmlBox())
     #~ def missed_reminders(cls,self,req):
@@ -567,9 +310,57 @@ class Home(mixins.EmptyTable):
           #~ max_items=10,before='<ul><li>',separator='</li><li>',after="</li></ul>")
           
     #~ @dd.constant('')
-    @dd.constant()
-    def welcome(cls,ui):
-        return "Welcome to the <b>%s</b> server." % cgi.escape(settings.LINO.title)
+    #~ @dd.constant()
+    #~ def welcome(cls,ui):
+        #~ return "Welcome to the <b>%s</b> server." % cgi.escape(settings.LINO.title)
+        
+    @dd.virtualfield(dd.HtmlBox(_('Welcome')))
+    def welcome(cls,self,ar):
+        #~ MAXITEMS = 2
+        u = ar.get_user()
+        story = []
+        
+        if u.profile.authenticated:
+          
+            intro = [_("Hi, "),u.first_name,'! ']
+            story.append(xghtml.E.p(*intro))
+            warnings = []
+            
+            #~ for T in (MySuggestedCoachings,cal.MyTasksToDo):
+            for table,text in settings.LINO.get_todo_tables(ar):
+                r = table.request(user=u)
+                #~ r = T.request(subst_user=u)
+                #~ r = ar.spawn(T)
+                if r.get_total_count() != 0:
+                    #~ for obj in r.data_iterator[-MAXITEMS]:
+                        #~ chunks = [obj.summary_row(ar)]
+                        #~ sep = ' : '
+                        #~ for a in T.get_workflow_actions(ar,obj):
+                            #~ chunks.append(sep)
+                            #~ chunks.append(ar.row_action_button(obj,a))
+                            #~ sep = ', '
+                    
+                    warnings.append(xghtml.E.li(
+                        ar.href_to_request(r,text % r.get_total_count())))
+                        #~ _("You have %d entries in ") % r.get_total_count(),
+                        #~ ar.href_to_request(r,label)))
+            
+            #~ warnings.append(xghtml.E.li("Test 1"))
+            #~ warnings.append(xghtml.E.li("Second test"))
+            if len(warnings):
+                #~ story.append(xghtml.E.h3(_("Warnings")))
+                story.append(xghtml.E.h3(_("You have")))
+                story.append(xghtml.E.ul(*warnings))
+            else:
+                story.append(xghtml.E.p(_("Congratulatons: you have no warnings.")))
+        #~ else:
+            # story.append(xghtml.E.p("Please log in"))
+            #~ story.append(settings.LINO.get_guest_greeting())
+        
+        return xghtml.E.div(*story,class_="htmlText",style="margin:5px")
+        
+
+        
     
     #~ @dd.virtualfield(dd.HtmlBox(_('Missed reminders')))
     #~ def missed_reminders(cls,self,req):
@@ -587,31 +378,48 @@ class Home(mixins.EmptyTable):
 
 
 
-
-def setup_main_menu(site,ui,user,m): pass
-
-def setup_my_menu(site,ui,user,m): pass
+SYSTEM_USER_LABEL = _("System")
+OFFICE_MODULE_LABEL = _("Office")
   
-def setup_config_menu(site,ui,user,m):
-    m = m.add_menu("etc",_("System"))
+def setup_main_menu(site,ui,profile,m): 
+    #~ office = m.add_menu("office",OFFICE_MODULE_LABEL)
+    #~ office.add_action(MyTextFieldTemplates)
+    pass
+
+def setup_my_menu(site,ui,profile,m): pass
+  
+def setup_config_menu(site,ui,profile,m):
+    office = m.add_menu("office",OFFICE_MODULE_LABEL)
+    system = m.add_menu("system",SYSTEM_USER_LABEL)
     #~ m.add_action('links.LinkTypes')
-    if site.user_model:
-        m.add_action(site.user_model)
-        m.add_action(site.modules.lino.TextFieldTemplates)
+    if site.user_model and profile.authenticated:
+        system.add_instance_action(site.site_config)
+        system.add_action(site.user_model)
+        office.add_action(MyTextFieldTemplates)
     #~ m.add_action(site.modules.users.Users)
     if site.is_installed('contenttypes'):
-        m.add_action(site.modules.lino.ContentTypes)
-        m.add_action(site.modules.lino.HelpTexts)
+        system.add_action(site.modules.lino.ContentTypes)
+        system.add_action(site.modules.lino.HelpTexts)
         #~ m.add_action(site.modules.lino.Workflows)
-    #~ if site.use_tinymce:
-    m.add_instance_action(site.site_config)
         
   
-def setup_explorer_menu(site,ui,user,m):
-    m.add_action(site.modules.users.Authorities)
+def setup_explorer_menu(site,ui,profile,m):
+    office = m.add_menu("office",OFFICE_MODULE_LABEL)
+    system = m.add_menu("system",SYSTEM_USER_LABEL)
+    if site.user_model:
+        system.add_action(site.modules.users.Authorities)
+        system.add_action(dd.UserGroups)
+        system.add_action(dd.UserLevels)
+        system.add_action(dd.UserProfiles)
+        office.add_action(TextFieldTemplates)
+    #~ if site.is_installed('contenttypes'):
+        #~ system.add_action(Changes)
   
-  
-def setup_site_menu(site,ui,user,m): 
-    m.add_action(site.modules.lino.About)
-    m.add_action(site.modules.lino.Inspector)
 
+dd.add_user_group('office',OFFICE_MODULE_LABEL)
+
+
+if settings.LINO.user_model == 'auth.User':
+    dd.inject_field(settings.LINO.user_model,'profile',dd.UserProfiles.field())
+    dd.inject_field(settings.LINO.user_model,'language',dd.LanguageField())
+    
\ No newline at end of file
