@@ -1,5 +1,5 @@
 # -*- coding: UTF-8 -*-
-## Copyright 2009-2012 Luc Saffre
+## Copyright 2009-2013 Luc Saffre
 ## This file is part of the Lino project.
 ## Lino is free software; you can redistribute it and/or modify 
 ## it under the terms of the GNU General Public License as published by
@@ -57,6 +57,7 @@ EXT_CHAR_HEIGHT = 22
 FULLWIDTH = '-20' 
 FULLHEIGHT = '-10' 
 
+USED_NUMBER_FORMATS = dict()
 
 #~ DEFAULT_GC_NAME = 'std'
 DEFAULT_GC_NAME = 0
@@ -156,8 +157,9 @@ class GridColumn(jsgen.Component):
     """
     declare_type = jsgen.DECLARE_INLINE
     
-    def __init__(self,index,editor,**kw):
-        """editor may be a Panel for columns on a GenericForeignKey
+    def __init__(self,layout_handle,index,editor,**kw):
+        """
+        editor may be a Panel for columns on a GenericForeignKey
         """
         #~ print 20100515, editor.name, editor.__class__
         #~ assert isinstance(editor,FieldElement), \
@@ -166,12 +168,15 @@ class GridColumn(jsgen.Component):
             #~ self.editor = None
         #~ else:
         self.editor = editor
-        #~ self.value_template = editor.grid_column_template
+        if editor.grid_column_template is not None:
+            self.value_template = editor.grid_column_template
         kw.update(sortable=True)
         #~ kw.update(submitValue=False) # 20110406
         kw.update(colIndex=index)
         kw.update(editor.get_column_options())
-        kw.update(hidden=editor.hidden)
+        #~ kw.update(hidden=editor.hidden)
+        if editor.hidden:
+            kw.update(hidden=True)
         if settings.LINO.use_filterRow:
             if editor.filter_type:
                 if index == 0:
@@ -194,16 +199,22 @@ class GridColumn(jsgen.Component):
                 kw.update(filter=editor.gridfilters_settings)
         #~ if isinstance(editor,FieldElement) and editor.field.primary_key:
         if isinstance(editor,FieldElement):
-            if settings.LINO.use_quicktips and self.editor.field.help_text:
-                #~ GridColumn tooltips don't support html
-                if not "<" in self.editor.field.help_text:
-                    kw.update(tooltip=self.editor.field.help_text)
+            if settings.LINO.use_quicktips:
+                #~ if jsgen._for_user_profile.expert:
+                if settings.LINO.show_internal_field_names:
+                    ttt = "(%s.%s) " % (layout_handle.layout._datasource,self.editor.field.name)
+                    #~ ttt = "(%s) " % self.editor.field.name
+                else:
+                    ttt = ''
+                if self.editor.field.help_text and not "<" in self.editor.field.help_text:
+                    #~ GridColumn tooltips don't support html
+                    ttt = string_concat(ttt,self.editor.field.help_text)
+                if ttt:
+                    kw.update(tooltip=ttt)
                 
             def fk_renderer(fld,name):
                 """
-                FK fields are clickable 
-                (1) if their target has a detail view
-                (2) 
+                FK fields are clickable only if their target has a detail view
                 """
                 rpt = fld.rel.to._lino_default_table
                 if rpt.detail_action is not None:
@@ -270,7 +281,7 @@ class Calendar(jsgen.Component):
     
 
 
-NOT_GIVEN = object()
+#~ NOT_GIVEN = object()
 
         
 class VisibleComponent(jsgen.Component,Permittable):
@@ -315,7 +326,7 @@ class VisibleComponent(jsgen.Component,Permittable):
         
     def setup(self,width=None,height=None,label=None,
         preferred_width=None,
-        required=NOT_GIVEN,
+        required=dd.NOT_PROVIDED,
         **kw):
         self.value.update(kw)
         #~ jsgen.Component.__init__(self,name,**kw)
@@ -327,7 +338,7 @@ class VisibleComponent(jsgen.Component,Permittable):
             self.height = height
         if label is not None:
             self.label = label
-        if required is not NOT_GIVEN:
+        if required is not dd.NOT_PROVIDED:
             self.required = required
             #~ if self.name == 'newcomers_left': # required.has_key('user_groups'):
                 #~ logger.info("20121130 setup() %s %s",self,self.required)
@@ -384,6 +395,7 @@ class LayoutElement(VisibleComponent):
     sortable = False
     xtype = None # set by subclasses
     #~ grid_column_template = "new Ext.grid.Column(%s)"
+    grid_column_template = None
     collapsible = False
     hidden = False
     active_child = True
@@ -548,6 +560,23 @@ class Spacer(LayoutElement):
     value_template = "new Ext.Spacer(%s)"
     
         
+def add_help_text(kw,help_text,title,datasource,fieldname):
+    if settings.LINO.use_quicktips:
+        if settings.LINO.show_internal_field_names:
+            ttt = "(%s.%s) " % (datasource,fieldname)
+        else:
+            ttt = ''
+        if help_text:
+            ttt = string_concat(ttt,help_text)
+        if ttt:
+            #~ kw.update(qtip=self.field.help_text)
+            #~ kw.update(toolTipText=self.field.help_text)
+            #~ kw.update(tooltip=self.field.help_text)
+            kw.update(listeners=dict(render=js_code(
+              "Lino.quicktip_renderer(%s,%s)" % (
+                  py2js(title),
+                  py2js(ttt)))
+            ))
         
         
 class FieldElement(LayoutElement):
@@ -567,17 +596,10 @@ class FieldElement(LayoutElement):
         self.field = field
         self.editable = field.editable # and not field.primary_key
         
-        if not isinstance(layout_handle.layout,layouts.ListLayout):
-            #~ help_text = getattr(self.field,'help_text',None):
-            if settings.LINO.use_quicktips and self.field.help_text:
-                #~ kw.update(qtip=self.field.help_text)
-                #~ kw.update(toolTipText=self.field.help_text)
-                #~ kw.update(tooltip=self.field.help_text)
-                kw.update(listeners=dict(render=js_code(
-                  "Lino.quicktip_renderer(%s,%s)" % (
-                      py2js(self.field.verbose_name),
-                      py2js(self.field.help_text)))
-                ))
+        if not kw.has_key('listeners'):
+            if not isinstance(layout_handle.layout,layouts.ListLayout):
+                add_help_text(kw,self.field.help_text,self.field.verbose_name,
+                  layout_handle.layout._datasource,self.field.name)
             
 
         #~ http://www.rowlands-bcs.com/extjs/tips/tooltips-form-fields
@@ -782,7 +804,6 @@ class CharFieldElement(FieldElement):
         #~ kw.update(margins='10px')
         return kw
         
-        
 class PasswordFieldElement(CharFieldElement):
     def get_field_options(self,**kw):
         kw = super(PasswordFieldElement,self).get_field_options(**kw)
@@ -956,6 +977,7 @@ class DatePickerFieldElement(FieldElement):
     
 class DateFieldElement(FieldElement):
     if settings.LINO.use_spinner:
+        raise Exception("20130114")
         value_template = "new Lino.SpinnerDateField(%s)"
     else:
         value_template = "new Lino.DateField(%s)"
@@ -1010,7 +1032,7 @@ class IntegerFieldElement(FieldElement):
     filter_type = 'numeric'
     gridfilters_settings = dict(type='numeric')
     #~ xtype = 'numberfield'
-    xtype = None
+    #~ xtype = None
     sortable = True
     preferred_width = 5
     #~ data_type = 'int' 
@@ -1036,12 +1058,13 @@ class IncompleteDateFieldElement(CharFieldElement):
 
 
 class DecimalFieldElement(FieldElement):
+    value_template = "new Ext.form.NumberField(%s)"
     filter_type = 'numeric'
     gridfilters_settings = dict(type='numeric')
-    xtype = 'numberfield'
+    #~ xtype = 'numberfield'
     sortable = True
     #~ data_type = 'float' 
-    #~ grid_column_template = "new Ext.grid.NumberColumn(%s)"
+    grid_column_template = "new Lino.NullNumberColumn(%s)"
     
     def __init__(self,*args,**kw):
         FieldElement.__init__(self,*args,**kw)
@@ -1052,31 +1075,18 @@ class DecimalFieldElement(FieldElement):
         kw = FieldElement.get_field_options(self,**kw)
         if self.field.decimal_places:
             kw.update(decimalPrecision=self.field.decimal_places)
+            #~ kw.update(decimalPrecision=-1)
             kw.update(decimalSeparator=settings.LINO.decimal_separator)
         else:
             kw.update(allowDecimals=False)
+        if self.editable:
+            kw.update(allowBlank=self.field.blank)
         return kw
         
-    #~ def get_column_options(self,**kw):
-        #~ kw = FieldElement.get_column_options(self,**kw)
-        #~ kw.update(xtype='numbercolumn')
-        #~ kw.update(align='right')
-        #~ if settings.LINO.decimal_separator == ',':
-            #~ fmt = "0.000"
-            #~ if self.field.decimal_places > 0:
-                #~ fmt += ',' + ("0" * self.field.decimal_places)
-                #~ fmt += "/i"
-        #~ elif settings.LINO.decimal_separator == '.':
-            #~ fmt = "0,000"
-            #~ if self.field.decimal_places > 0:
-                #~ fmt += '.' + ("0" * self.field.decimal_places)
-        #~ kw.update(format=fmt)
-        #~ return kw
-        
     def get_column_options(self,**kw):
         kw = FieldElement.get_column_options(self,**kw)
-        kw.update(xtype='numbercolumn')
-        kw.update(align='right')
+        #~ kw.update(xtype='numbercolumn')
+        #~ kw.update(align='right')
         #~ if settings.LINO.decimal_group_separator:
             #~ fmt = '0' + settings.LINO.decimal_group_separator + '000'
         #~ else:
@@ -1087,9 +1097,34 @@ class DecimalFieldElement(FieldElement):
             fmt += settings.LINO.decimal_separator + ("0" * self.field.decimal_places)
         if settings.LINO.decimal_separator == ',':
             fmt += "/i"
-        kw.update(format=fmt)
+        if fmt != settings.LINO.default_number_format_extjs:
+            kw.update(format=fmt)
+        n = USED_NUMBER_FORMATS.get(fmt,0)
+        USED_NUMBER_FORMATS[fmt] = n + 1
+        #~ kw.update(format='') # 20130125
+        #~ kw.update(renderer=js_code('Lino.nullnumbercolumn_renderer')) # 20130125
         return kw
         
+class QuantityFieldElement(CharFieldElement):
+#~ class QuantityFieldElement(DecimalFieldElement):
+    #~ grid_column_template = "new Lino.NullNumberColumn(%s)"
+    #~ def get_field_options(self,**kw):
+        #~ kw = CharFieldElement.get_field_options(self,**kw)
+        #~ kw.update(align='right')
+        #~ kw.update(fieldClass="x-form-field x-form-num-field")
+        #~ return kw
+  
+    def get_column_options(self,**kw):
+        #~ print 20130125, self.field.name
+        kw = super(QuantityFieldElement,self).get_column_options(**kw)
+        #~ kw.update(xtype='numbercolumn')
+        kw.update(align='right')
+        kw.update(format='') # 20130125
+        #~ kw.update(renderer=js_code('Lino.nullnumbercolumn_renderer')) # 20130125
+        return kw
+        
+
+
 class DisplayElement(FieldElement):
     """
     ExtJS element to be used for :class:`DisplayFields <lino.core.fields.DisplayField>`.
@@ -1756,7 +1791,7 @@ class GridElement(Container):
     def __init__(self,layout_handle,name,rpt,*columns,**kw):
         """
         :param layout_handle: the handle of the FormLayout owning this grid
-        :param rpt: the report being displayed
+        :param rpt: the table being displayed (:class:`lino.core.tables.AbstractTable`)
         """
         #~ assert isinstance(rpt,dd.AbstractTable), "%r is not a Table!" % rpt
         self.value_template = "new Lino.%s.GridPanel(%%s)" % rpt
@@ -1786,9 +1821,15 @@ class GridElement(Container):
         #~ //~ scrollOffset:200,
         #~ //~ enableRowBody: true,
         
+        if rpt.auto_fit_column_widths:
+            vc.update(forceFit=True)
+            
         kw.update(viewConfig=vc)
         kw.setdefault('label',rpt.label)
         
+        add_help_text(kw,rpt.help_text,rpt.title or rpt.label,rpt.app_label,rpt.actor_id)
+        
+        
         #~ kw.update(containing_window=js_code("this.containing_window"))
         kw.update(containing_panel=js_code("this"))
         #~ if not rpt.show_params_at_render:
@@ -1916,6 +1957,7 @@ _FIELD2ELEM = (
     (dd.HtmlBox, HtmlBoxElement),
     #~ (dd.QuickAction, QuickActionElement),
     (dd.DisplayField, DisplayElement),
+    (dd.QuantityField, QuantityFieldElement),
     (dd.IncompleteDateField, IncompleteDateFieldElement),
     #~ (dd.LinkedForeignKey, LinkedForeignKeyElement),
     (models.URLField, URLFieldElement),
@@ -1933,7 +1975,6 @@ _FIELD2ELEM = (
     (models.TimeField, TimeFieldElement),
     (models.IntegerField, IntegerFieldElement),
     (models.DecimalField, DecimalFieldElement),
-    (dd.QuantityField, CharFieldElement),
     (models.BooleanField, BooleanFieldElement),
     #~ (models.ManyToManyField, M2mGridElement),
     (models.ForeignKey, ForeignKeyElement),
