@@ -1,5 +1,5 @@
 # -*- coding: UTF-8 -*-
-## Copyright 2009-2012 Luc Saffre
+## Copyright 2009-2013 Luc Saffre
 ## This file is part of the Lino project.
 ## Lino is free software; you can redistribute it and/or modify 
 ## it under the terms of the GNU General Public License as published by
@@ -30,6 +30,7 @@ from django.utils.translation import ugettext as _
 from django.utils.encoding import force_unicode
 
 from lino import dd
+#~ from lino.core.signals import pre_ui_delete
 
 from lino.utils.xmlgen import html as xghtml
 E = xghtml.E
@@ -46,7 +47,7 @@ from lino.utils import auth
 from lino.core import actions
 from lino.core import actors
 from lino.core import dbtables
-from lino.core import changes
+#~ from lino.core import changes
 from lino.core import web
 
 from lino.core.modeltools import obj2str, obj2unicode
@@ -72,6 +73,9 @@ def requested_actor(app_label,actor):
         return cl._lino_default_table
     return cl
     
+#~ class Http403(Exception):
+    #~ pass
+    
 def action_request(app_label,actor,request,rqdata,is_list,**kw):
     rpt = requested_actor(app_label,actor)
     action_name = rqdata.get(ext_requests.URL_PARAM_ACTION_NAME,None)
@@ -86,7 +90,10 @@ def action_request(app_label,actor,request,rqdata,is_list,**kw):
             rpt,action_name,rpt.get_url_action_names()))
     user = request.subst_user or request.user
     if not a.get_view_permission(user.profile):
-        raise Exception("Action not allowed for %s" % user)
+        #~ raise Http403(_("As %s you have no permission to run this action.") % user.profile)
+        raise exceptions.PermissionDenied(
+            _("As %s you have no permission to run this action.") % user.profile)
+        #~ return http.HttpResponseForbidden(_("As %s you have no permission to run this action.") % user.profile)
     ar = rpt.request(settings.LINO.ui,request,a,**kw)
     #~ ar.renderer = settings.LINO.ui.ext_renderer
     return ar
@@ -116,13 +123,13 @@ def run_action(ar,elem):
           #~ message=unicode(e),
           #~ decide_id=ar.ui.add_decision(e))
         #~ return ar.ui.action_response(r)
-    except Warning,e:
+    except Warning as e:
         r = dict(
           success=False,
           message=unicode(e),
           alert=True)
         return ar.ui.action_response(r)
-    except Exception,e:
+    except Exception as e:
         if elem is None:
             msg = unicode(e)
         else:
@@ -217,6 +224,9 @@ def elem2rec_detailed(ar,elem,**rec):
     if ar.actor.hide_top_toolbar:
         rec.update(title=unicode(elem))
     else:
+        #~ print(ar.get_title())
+        #~ print(obj2str(elem))
+        #~ print(repr(unicode(elem)))
         rec.update(title=ar.get_title() + u" » " + unicode(elem))
     rec.update(id=elem.pk)
     rec.update(disable_delete=rh.actor.disable_delete(elem,ar))
@@ -276,7 +286,9 @@ def delete_element(ar,elem):
             
     #~ dblogger.log_deleted(ar.request,elem)
     
-    changes.log_delete(ar.request,elem)
+    #~ changes.log_delete(ar.request,elem)
+    
+    dd.pre_ui_delete.send(sender=elem,request=ar.request)
     
     try:
         elem.delete()
@@ -292,10 +304,16 @@ def delete_element(ar,elem):
     
     return HttpResponseDeleted()
     
+CATCHED_AJAX_EXCEPTIONS = (Warning,IntegrityError,exceptions.ValidationError)
+
+def ajax_error(e,**kw):
+    kw = settings.LINO.ui.error(e,alert=True,**kw)
+    return json_response(kw)
 
 #~ def form2obj_and_save(self,request,rh,data,elem,is_new,include_rows): # **kw2save):
 def form2obj_and_save(ar,data,elem,is_new,restful,file_upload=False): # **kw2save):
     """
+    Parses the data from HttpRequest to the model instance and saves it.
     """
     #~ self = settings.LINO.ui
     request = ar.request
@@ -307,23 +325,18 @@ def form2obj_and_save(ar,data,elem,is_new,restful,file_upload=False): # **kw2sav
     # store normal form data (POST or PUT)
     #~ original_state = dict(elem.__dict__)
     if not is_new:
-        watcher = changes.Watcher(elem)
+        watcher = dd.ChangeWatcher(elem)
     try:
         rh.store.form2obj(ar,data,elem,is_new)
-    except exceptions.ValidationError,e:
-        #~ raise
-        kw = settings.LINO.ui.error(e)
-        return json_response(kw)
-       #~ return error(e,_("There was a problem while validating your data : "))
-    #~ logger.info('20120228 store.form2obj passed, elem is %s' % obj2str(elem))
-    
+        elem.full_clean()
+    except CATCHED_AJAX_EXCEPTIONS as e:
+        return ajax_error(e)
+        
     kw = dict(success=True)
     
-    try:
-        elem.full_clean()
-    except exceptions.ValidationError, e:
-        kw = settings.LINO.ui.error(e) #,_("There was a problem while validating your data : "))
-        return json_response(kw)
+    #~ except exceptions.ValidationError, e:
+        #~ kw = settings.LINO.ui.error(e) 
+        #~ return json_response(kw)
     
     #~ dirty = False
     #~ missing = object()
@@ -347,20 +360,20 @@ def form2obj_and_save(ar,data,elem,is_new,restful,file_upload=False): # **kw2sav
             
         try:
             elem.save(**kw2save)
-        #~ except Exception,e:
-        except (Warning,IntegrityError),e:
-            kw = settings.LINO.ui.error(e,alert=True) # ,_("There was a problem while saving your data : "))
+        except CATCHED_AJAX_EXCEPTIONS,e:
+            return ajax_error(e)
             #~ return views.json_response_kw(success=False,
                   #~ msg=_("There was a problem while saving your data:\n%s") % e)
-            return json_response(kw)
                   
         if is_new:
-            changes.log_create(request,elem)
+            dd.pre_ui_create.send(elem,request=request)
+            #~ changes.log_create(request,elem)
             kw.update(
                 message=_("%s has been created.") % obj2unicode(elem))
                 #~ record_id=elem.pk)
         else:
-            watcher.log_diff(request)
+            watcher.send_update(request)
+            #~ watcher.log_diff(request)
             kw.update(message=_("%s has been updated.") % obj2unicode(elem))
         
     else:
@@ -528,7 +541,7 @@ def choices_for_field(request,rpt,field):
             def row2dict(obj,d):
                 d[ext_requests.CHOICES_TEXT_FIELD] = unicode(obj)
                 #~ 20130117 : 'int' object has no attribute 'get_choices_text'
-                #~ d[ext_requests.CHOICES_VALUE_FIELD] = obj.get_choices_text(request,rpt,field)
+                d[ext_requests.CHOICES_VALUE_FIELD] = obj
                 return d
         elif chooser.instance_values:
             # same code as for ForeignKey
@@ -717,7 +730,7 @@ class Restful(View):
         ar = rpt.request(ui,request)
         rh = ar.ah
             
-        data = http.QueryDict(request.raw_post_data).get('rows')
+        data = http.QueryDict(request.body).get('rows') # raw_post_data before Django 1.4
         data = json.loads(data)
         a = rpt.get_url_action(rpt.default_list_action_name)
         ar = rpt.request(ui,request,a)
@@ -819,10 +832,9 @@ class ApiElement(View):
         raise NotImplementedError("Action %s is not implemented)" % ar)
         
     def put(self,request,app_label=None,actor=None,pk=None):
-        data = http.QueryDict(request.raw_post_data)
+        data = http.QueryDict(request.body) # raw_post_data before Django 1.4
         ar = action_request(app_label,actor,request,data,False)
         ar.renderer = settings.LINO.ui.ext_renderer
-        
         elem = ar.actor.get_row_by_pk(pk)
         if elem is None:
             raise http.Http404("%s has no row with primary key %r" % (rpt,pk))
@@ -1024,7 +1036,7 @@ class GridConfig(View):
         ui = settings.LINO.ui
         rpt = requested_actor(app_label,actor)
         #~ rpt = actors.get_actor2(app_label,actor)
-        PUT = http.QueryDict(request.raw_post_data)
+        PUT = http.QueryDict(request.body) # raw_post_data before Django 1.4
         gc = dict(
           widths = [int(x) for x in PUT.getlist(ext_requests.URL_PARAM_WIDTHS)],
           columns = [str(x) for x in PUT.getlist(ext_requests.URL_PARAM_COLUMNS)],
@@ -1053,12 +1065,20 @@ class GridConfig(View):
         #~ logger.info(msg)
         settings.LINO.ui.build_site_cache(True)            
         return settings.LINO.ui.success(msg)
+        
+MENUS = dict()        
             
 def plain_response(ui,request,tplname,context):        
     u = request.subst_user or request.user
-    menu = settings.LINO.get_site_menu(ui,u.profile)
-    menu = menu.as_html(ui,request)
-    context.update(menu=E.tostring(menu))
+    menu = MENUS.get(u.profile,None)
+    if menu is None:
+        menu = settings.LINO.get_site_menu(ui,u.profile)
+        url = settings.LINO.plain_prefix + '/'
+        menu.add_url_button(url,label=_("Home"))
+        menu = menu.as_html(ui,request)
+        menu = E.tostring(menu)
+        MENUS[u.profile] = menu
+    context.update(menu=menu)
     web.extend_context(context)
     template = settings.LINO.jinja_env.get_template(tplname)
     
@@ -1128,7 +1148,7 @@ class PlainIndex(View):
         a = settings.LINO.get_main_action(user)
         if a is not None:
             if not a.get_view_permission(user.profile):
-                raise Exception("Action not allowed for %s" % user)
+                raise PermissionDenied("Action not allowed for %s" % user)
             ar = a.request(settings.LINO.ui,request,**kw)
             ar.renderer = ui.plain_renderer
             context.update(title=ar.get_title())
